#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Data;
using System.Xml;
using System.Diagnostics;

namespace HeuristicLab.Constraints {
  public class AllSubOperatorsTypeConstraint : ConstraintBase {

    private SubOperatorTypeConstraint groupConstraint;
    public IList<IOperator> AllowedSubOperators {
      get {
        return groupConstraint.AllowedSubOperators;
      }
    }

    public override string Description {
      get { return "All sub-operators have to be elements of a pre-defined group."; }
    }

    /// <summary>
    /// Default constructor for an empty AllSubOperatorsTypeConstraint.
    /// </summary>
    public AllSubOperatorsTypeConstraint() {
      groupConstraint = new SubOperatorTypeConstraint();
    }
    /// <summary>
    /// Copy constructor to create clones (deep).
    /// </summary>
    /// <param name="original">The original instance to be cloned.</param>
    public AllSubOperatorsTypeConstraint(AllSubOperatorsTypeConstraint original)
      : this(original, new Dictionary<Guid, object>()) { }
    /// <summary>
    /// Copy constructor to create clones (deep) reusing already cloned object references.
    /// </summary>
    /// <param name="original">The instance to be cloned.</param>
    /// <param name="clonedObjects">Already cloned objects (for referential integrity).</param>
    protected AllSubOperatorsTypeConstraint(AllSubOperatorsTypeConstraint original, IDictionary<Guid, object> clonedObjects)
      : base(original, clonedObjects) {
      this.groupConstraint = (SubOperatorTypeConstraint)Auxiliary.Clone(original.groupConstraint, clonedObjects);
    }

    public void AddOperator(IOperator op) {
      groupConstraint.AddOperator(op);
      FireChanged();
    }

    public void RemoveOperator(IOperator op) {
      groupConstraint.RemoveOperator(op);
      FireChanged();
    }

    public override bool Check(IItem data) {
      IOperator op = data as IOperator;
      if (data == null) return false;

      for (int i = 0; i < op.SubOperators.Count; i++) {
        groupConstraint.SubOperatorIndex.Data = i;
        if (groupConstraint.Check(data) == false) {
          return false;
        }
      }
      return true;
    }

    public void Clear() {
      groupConstraint.Clear();
    }

    public override object Clone(IDictionary<Guid, object> clonedObjects) {
      return new AllSubOperatorsTypeConstraint(this);
    }

    public override IView CreateView() {
      return new AllSubOperatorsTypeConstraintView(groupConstraint);
    }

    #region persistence
    public override XmlNode GetXmlNode(string name, XmlDocument document, IDictionary<Guid, IStorable> persistedObjects) {
      XmlNode node = base.GetXmlNode(name, document, persistedObjects);
      XmlNode subOperatorsNode = PersistenceManager.Persist("SubOperatorsGroupConstraint", groupConstraint, document, persistedObjects);
      node.AppendChild(subOperatorsNode);

      return node;
    }

    public override void Populate(XmlNode node, IDictionary<Guid, IStorable> restoredObjects) {
      base.Populate(node, restoredObjects);
      groupConstraint = (SubOperatorTypeConstraint)PersistenceManager.Restore(node.SelectSingleNode("SubOperatorsGroupConstraint"), restoredObjects);
    }
    #endregion persistence
  }
}
